//
// VMime library (http://www.vmime.org)
// Copyright (C) 2002 Vincent Richard <vincent@vmime.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 3 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Linking this library statically or dynamically with other modules is making
// a combined work based on this library.  Thus, the terms and conditions of
// the GNU General Public License cover the whole combination.
//

#ifndef VMIME_NET_TLSSECUREDCONNECTIONINFOS_HPP_INCLUDED
#define VMIME_NET_TLSSECUREDCONNECTIONINFOS_HPP_INCLUDED


#include "vmime/config.hpp"


#if VMIME_HAVE_MESSAGING_FEATURES && VMIME_HAVE_TLS_SUPPORT


#include "vmime/net/securedConnectionInfos.hpp"

#include "vmime/security/cert/certificateChain.hpp"


namespace vmime {
namespace net {
namespace tls {


class TLSSession;
class TLSSocket;


/** Information about a TLS-secured connection used by a service.
  */
class VMIME_EXPORT TLSSecuredConnectionInfos : public securedConnectionInfos {

public:

	TLSSecuredConnectionInfos(
		const string& host,
		const port_t port,
		const shared_ptr <TLSSession>& tlsSession,
		const shared_ptr <TLSSocket>& tlsSocket
	);

	const string getHost() const;
	port_t getPort() const;

	/** Return the peer's certificate (chain) as sent by the peer.
	  *
	  * @return server certificate chain
	  */
	shared_ptr <const security::cert::certificateChain> getPeerCertificates() const;

private:

	string m_host;
	port_t m_port;

	shared_ptr <TLSSession> m_tlsSession;
	shared_ptr <TLSSocket> m_tlsSocket;
};


} // tls
} // net
} // vmime


#endif // VMIME_HAVE_MESSAGING_FEATURES && VMIME_HAVE_TLS_SUPPORT

#endif // VMIME_NET_TLSSECUREDCONNECTIONINFOS_HPP_INCLUDED

